@@ -39,7 +39,7 @@ sub configure_nqp {
         if ( defined $passed_backends && $nqp_backend ne $passed_backends ) {
             $self->sorry(
                 "Passed value to --backends ($passed_backends) is overwritten ",
-                "by the one infered by --with-nqp ($nqp_backend)"
+                "by the one inferred by --with-nqp ($nqp_backend)"
             );
         }
     }
@@ -161,12 +161,23 @@ sub configure_refine_vars {
         )
     );
 
+    $config->{nqp_home} = $self->nfp(
+        File::Spec->rel2abs(
+            $config->{nqp_home}
+              || File::Spec->catdir( $config->{'prefix'}, 'share', 'nqp' )
+        )
+    );
+
     $config->{static_rakudo_home} =
       $config->{relocatable} eq 'reloc'
       ? ''
       : $config->{rakudo_home};
 }
 
+sub rev2spec {
+    chr(98 + $_[1]); # 98 stands for 'b'
+}
+
 sub parse_lang_specs {
     my $self = shift;
 
@@ -175,7 +186,8 @@ sub parse_lang_specs {
     open my $sh, "<", $self->template_file_path( $tmpl, required => 1 )
       or self->sorry("Can't open $tmpl: $!");
 
-    my @specs;    # Array to preserve the order or specs
+    my @revs;    # Array to preserve the order or specs
+    my @specs;   # Perl6 spec letters
     my $ln        = 0;
     my %flag_name = ( '-' => 'deprecate', '!' => 'require', );
     my $fail      = sub {
@@ -186,14 +198,15 @@ sub parse_lang_specs {
         chomp $line;
         $line =~ s/\h*#.*$//;    # Cut off comment
         next unless $line;
-        if ( $line =~ s/^\h*(?<default>\*)?(?<letter>[c-z])\b// ) {
-            my $letter = $+{letter};
+        if ( $line =~ s/^\h*(?<default>\*)?(?<number>\d+)\b// ) {
+            my $number = $+{number};
             if ( $+{default} ) {
-                $fail->("duplicate default spec") if $config->{lang_spec};
-                $config->{lang_spec} = $letter;
+                $fail->("duplicate default spec") if $config->{lang_rev};
+                $config->{lang_rev} = $number;
             }
-            my @def = ($letter);
-            push @specs, \@def;
+            my @def = ($number);
+            push @revs, \@def;
+            push @specs, $self->rev2spec($number);
             my @mods;            # Array to preserve the order of modificators
             while ($line) {
                 unless ( $line =~ s/^\h+// ) {
@@ -214,10 +227,11 @@ sub parse_lang_specs {
             push @def, @mods;
         }
         else {
-            $fail->("expected a revision letter");
+            $fail->("expected a revision number");
         }
     }
 
+    $self->{raku_revisions} = \@revs;
     $self->{raku_specs} = \@specs;
 
     close $sh;
@@ -236,23 +250,6 @@ sub configure_misc {
 
     $self->parse_lang_specs;
 
-    #my $spec_line = sub {
-    #    my @elems = split ' ', shift;
-    #    if ( $elems[0] =~ s/^\*// ) {
-    #        $config->{lang_spec} = $elems[0];
-    #    }
-    #    return \@elems;
-    #};
-
-    #$self->{raku_specs} = [
-    #    map { $spec_line->($_) }
-    #      grep { s/\s*#.*$//; length }
-    #      split(
-    #        /\n/s,
-    #        slurp( $self->template_file_path( 'RAKU_SPECS', required => 1, ) )
-    #      )
-    #];
-
     # Get version info from VERSION template and git.
     my $VERSION = slurp( File::Spec->catfile( $self->cfg('base_dir'), 'VERSION' ) );
     chomp $VERSION;
@@ -383,6 +380,7 @@ sub configure_moar_backend {
           );
     }
     $nqp_config->{mingw_unicode} = '';
+    $nqp_config->{subsystem_windows_flag} = '';
 
     my @c_runner_libs;
     if ( $self->is_win ) {
@@ -402,7 +400,17 @@ sub configure_moar_backend {
         if ( $nqp_config->{'moar::os'} eq 'mingw32' ) {
             $nqp_config->{mingw_unicode} = '-municode';
         }
+
+        $nqp_config->{subsystem_win_cc_flags} = '-DSUBSYSTEM_WINDOWS';
+        if ( $nqp_config->{'moar::ld'} eq 'link' ) {
+            $nqp_config->{subsystem_win_ld_flags} = '/subsystem:windows';
+        }
+        else {
+            $nqp_config->{subsystem_win_ld_flags} = '-mwindows';
+        }
+
         push @c_runner_libs, sprintf( $nqp_config->{'moar::ldusr'}, 'Shlwapi' );
+        push @c_runner_libs, sprintf( $nqp_config->{'moar::ldusr'}, 'Shell32' );
     }
     else {
         $imoar->{toolchains} = [qw<gdb lldb valgrind>];
@@ -537,12 +545,17 @@ sub clean_old_p6_libs {
     }
 }
 
-# Returns all active language specification entries except for .c
+# Returns all active language specification 
 sub raku_specs {
     my $self = shift;
     return @{ $self->{raku_specs} };
 }
 
+sub raku_revisions {
+    my $self = shift;
+    return @{ $self->{raku_revisions} };
+}
+
 sub post_active_backends {
     my $self = shift;
 
@@ -636,19 +649,17 @@ sub gen_nqp {
 
         unless ( $force_rebuild
             || !$nqp_have
-            || $nqp_ver_ok
-            || $options->{'ignore-errors'} )
+            || $nqp_ok
+            || defined($gen_nqp))
         {
-            $self->note( "WARNING",
+            my $say_sub = $options->{'ignore-errors'} ? 'note' : 'sorry';
+            $self->$say_sub(
                 "$bin version $nqp_have is outdated, $nqp_want expected.\n" );
         }
 
         if ( !$force_rebuild and ( $nqp_ok or $options->{'ignore-errors'} ) ) {
             $impls->{$b}{ok} = 1;
         }
-        elsif ( $self->opt('with-nqp') ) {
-            $self->sorry("$bin version cannot be used");
-        }
         else {
             $need{$b} = 1;
         }
@@ -658,8 +669,8 @@ sub gen_nqp {
 
     return unless defined($gen_nqp) || defined($gen_moar);
 
-    if ( defined $gen_nqp || defined $gen_moar ) {
-        my $user = $options->{'github-user'} // 'perl6';
+    {
+        my $user = $options->{'github-user'} // 'Raku';
 
         # Don't expect any specific default commit in nqp/ if the repo is
         # already checked out and we're force-rebuilding.
@@ -717,7 +728,7 @@ use warnings;
 use File::Find;
 
 # --- Rakudo-specific macro methods.
-sub _specs_iterate {
+sub _revs_iterate {
     my $self   = shift;
     my $cb     = shift;
     my %params = @_;
@@ -726,28 +737,39 @@ sub _specs_iterate {
 
     $self->not_in_context( specs => 'spec' );
 
-    for my $spec ( $cfg->raku_specs ) {
-        my $spec_char   = $spec->[0];
+    my $prev_rev_num;
+    for my $rev ( $cfg->raku_revisions ) {
+        my $rev_num   = $rev->[0];
+        $prev_rev_num //= $rev_num; # Map 1 -> 1
+        my $spec_char = $cfg->rev2spec($rev_num);
+        my $prev_spec_char = $cfg->rev2spec($prev_rev_num);
         my $spec_subdir = "6.$spec_char";
         my %config      = (
+            lang_rev      => $rev_num,
+            prev_lang_rev => $prev_rev_num,
+            lrev_with_mod => $rev_num,
             ctx_subdir    => $spec_subdir,
             spec_subdir   => $spec_subdir,
             spec          => $spec_char,
             spec_with_mod => $spec_char,
             ucspec        => uc $spec_char,
             lcspec        => lc $spec_char,
+            prevspec      => $prev_spec_char,
+            ucprevspec    => uc $prev_spec_char,
+            lcprevspec    => lc $prev_spec_char,
         );
-        my $spec_ctx = {
-            spec    => $spec,
+        my $rev_ctx = {
+            rev     => $rev,
             configs => [ \%config ],
         };
-        my $s = $cfg->push_ctx($spec_ctx);
+        my $s = $cfg->push_ctx($rev_ctx);
         $cb->(@_);
-        if ( $params{with_mods} && @$spec > 1 ) {
-            for my $mod ( @$spec[ 1 .. $#$spec ] ) {
+        if ( $params{with_mods} && @$rev > 1 ) {
+            for my $mod ( @$rev[ 1 .. $#$rev ] ) {
                 my %mod = (
-                    spec_mod      => $mod->[0],
-                    spec_dot_mod  => ".$mod->[0]",
+                    lrev_mod      => $mod->[0],
+                    lrev_dot_mod  => ".$mod->[0]",
+                    lrev_with_mod => "$rev_num.$mod->[0]",
                     spec_with_mod => "$spec_char.$mod->[0]",
                 );
                 my $mod_s = $cfg->push_ctx(
@@ -758,6 +780,7 @@ sub _specs_iterate {
                 $cb->(@_);
             }
         }
+        $prev_rev_num = $rev_num;
     }
 }
 
@@ -775,11 +798,10 @@ sub _specs_iterate {
                     push @all_sources,
                       File::Spec->abs2rel(
                         File::Spec->catfile( $File::Find::dir, $_ ), $base_dir )
-                      if /\.(nqp|pm6)\z/;
+                      if /\.(nqp|pm6|rakumod)\z/;
                 },
                 File::Spec->catdir( $base_dir, "src" )
             );
-            push @all_sources, 'gen/nqp-version';
         }
         return @all_sources;
     }
@@ -787,9 +809,9 @@ sub _specs_iterate {
 
 # --- Macro implementations
 
-# for_specs(text)
-# Iterates over active backends and expands text in the context of each backend.
-sub _m_for_specs {
+# for_specs(text), for_revs(texT)
+# Iterates over active language revisions from RAKU_SPEC and expands text in the context of each revision.
+sub _m_for_revs {
     my $self   = shift;
     my $text   = shift;
     my %params = @_;
@@ -800,17 +822,17 @@ sub _m_for_specs {
         $out .= $self->_expand($text);
     };
 
-    _specs_iterate( $self, $cb, %params );
+    _revs_iterate( $self, $cb, %params );
 
     return $out;
 }
 
-# for_specmods(text)
-# Same as for_specs but iterates including spec modifications, i.e. 6.x.PREVIEW
-sub _m_for_specmods {
+# for_specmods(text), for_revmods(text)
+# Same as for_revs but iterates including modifications, i.e. <rev>.PREVIEW
+sub _m_for_revmods {
     my $self = shift;
     my $text = shift;
-    return _m_for_specs( $self, $text, with_mods => 1 );
+    return _m_for_revs( $self, $text, with_mods => 1 );
 }
 
 # for_toolchain(text)
@@ -872,7 +894,7 @@ sub _m_for_langalias {
 
 sub _m_source_digest_files {
     my $self   = shift;
-    my $indent = " " x ( $self->cfg->{config}{filelist_indent} || 4 );
+    my $indent = " " x ( $self->cfg->{config}{list_indent} || 4 );
     return join( " \\\n$indent", _all_sources($self) );
 }
 
@@ -916,8 +938,10 @@ $text
 TPL
 }
 
-NQP::Macros->register_macro( 'for_specs',           \&_m_for_specs );
-NQP::Macros->register_macro( 'for_specmods',        \&_m_for_specmods );
+NQP::Macros->register_macro( 'for_specs',           \&_m_for_revs );
+NQP::Macros->register_macro( 'for_revs',            \&_m_for_revs );
+NQP::Macros->register_macro( 'for_specmods',        \&_m_for_revmods );
+NQP::Macros->register_macro( 'for_revmods',         \&_m_for_revmods );
 NQP::Macros->register_macro( 'for_toolchain',       \&_m_for_toolchain );
 NQP::Macros->register_macro( 'for_langalias',       \&_m_for_langalias );
 NQP::Macros->register_macro( 'source_digest',       \&_m_source_digest );
